package lite

import (
	"encoding/json"
	//"errors"
	"fmt"
	"strconv"
	. "template"
	"time"

	simplejson "github.com/go-simplejson"
	tml "github.com/toml-master"
)

var (
	BucketLTC  string = "LTC"
	LiteConfig config
	liteOp     LiteApi
)

type Lite struct {
	walletstatus bool
}

const (
	NameLite = "LTC"
)

func (m Lite) InitDriver(parm interface{}) bool {
	if m.initconfig() == false {
		return false
	}
	go m.getwalletinfo()
	return true
}

//get the server's total available balance
func (m Lite) CoinBalance(parm interface{}) ([]byte, error) {

	confirmedBalance, err := liteOp.WalleGetBalance("*")
	if err != nil {
		LOG("ERROR", "lite GetBalance err:%v", err.Error())
		return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, GETBALANCE_ERR, getErrMsg(GETBALANCE_ERR, nil))
	}

	UnconfirmedBalance, err := liteOp.WalleGetUnconfirmedBalance("")
	if err != nil {
		LOG("ERROR", "lite GetUnconfirmedBalance err:%s", err.Error())
		return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, GETUNCONFIRMEDBALANCE_ERR, getErrMsg(GETUNCONFIRMEDBALANCE_ERR, nil))
	}

	var jbuf []byte
	resp := make(map[string]interface{})
	resp["confirmedbalance"] = confirmedBalance
	resp["unconfirmed"] = UnconfirmedBalance
	resp["errcode"] = 0
	resp["exact"] = 0

	if jbuf, err = json.Marshal(resp); err != nil {
		LOG("ERROR", "lite inner json error:%v", err.Error())
		return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, INNER_ERR, getErrMsg(INNER_ERR, nil))
	}

	return jbuf, nil
}

//Returns up to 'count' most recent transactions skipping the first 'from' transactions for account 'account'.
func (m Lite) TransInfo(parm interface{}) ([]byte, error) {

	pjs := parm.(*simplejson.Json)
	startH := pjs.Get("starth").MustUint64()
	endH := pjs.Get("endh").MustUint64()

	if startH < 1 {
		LOG("DEBUG", "lite input para  err:%v", startH)
		startH = 1
	}

	//get longest block count
	blockcount, err := liteOp.WalleGetBlockCount()
	if err != nil {
		LOG("ERROR", "lite WalleGetBlockCount  err:%v", err.Error())
		return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, GETCLOCKCOUNT_ERR, getErrMsg(GETCLOCKCOUNT_ERR, nil))
	}

	//get startblock hash by height

	if startH > blockcount {
		LOG("DEBUG", "lite startheight:%v >  blockcount:%v", startH, blockcount)
		startH = blockcount
	}

	startblockhash, err := liteOp.WalletGetBlockHash(uint64(startH - 1))
	if err != nil {
		LOG("ERROR", "lite WalletGetBlockHash startH:%v err:%v", startH, err.Error())
		return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, GETBLOCKINFO_ERR, getErrMsg(GETBLOCKINFO_ERR, nil))
	}

	//get endblock time by height
	if endH > blockcount {
		LOG("DEBUG", "lite endheight:%v >  blockcount:%v", endH, blockcount)
		endH = blockcount
	}

	endblockhash, err := liteOp.WalletGetBlockHash(uint64(endH))
	if err != nil {
		LOG("ERROR", "lite WalletGetBlockHash startH:%v err:%v", startH, err.Error())
		return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, GETBLOCKINFO_ERR, getErrMsg(GETBLOCKINFO_ERR, nil))
	}

	endblocktime, err := liteOp.WalletGetBlockTime(endblockhash)
	if err != nil {
		LOG("ERROR", "lite WalletGetBlockTime endH:%v err:%v", endH, err.Error())
		return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, GETBLOCKINFO_ERR, getErrMsg(GETBLOCKINFO_ERR, nil))
	}

	//Get all transactions in blocks since block [blockhash]
	resp, err := liteOp.WalleListSinceBlock(startblockhash)
	if err != nil {
		LOG("ERROR", "lite WalleListSinceBlock err:%v", err.Error())
		return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, TRANSINFO_ERR, getErrMsg(TRANSINFO_ERR, nil))
	}

	var litelisttransactionsinfo []Listtransactions

	if err = json.Unmarshal([]byte(resp), &litelisttransactionsinfo); err != nil {
		LOG("ERROR", "lite inner json error:%v", err.Error())
		return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, INNER_ERR, getErrMsg(INNER_ERR, nil))
	}

	outputsArr := make([]map[string]interface{}, 0)

	for _, listtransaction := range litelisttransactionsinfo {

		// get all transactions from block startHright to endHeight
		if listtransaction.Blocktime <= endblocktime {
			requestData := make(map[string]interface{})
			confirmationheight, err := liteOp.WalletGetBlockHeight(listtransaction.Blockhash)
			if err != nil {
				LOG("ERROR", "WalletGetBlockHeight error:%v", err.Error())
				return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, GETBLOCKINFO_ERR, getErrMsg(GETBLOCKINFO_ERR, nil))
			}
			requestData["confirmationheight"] = confirmationheight
			requestData["address"] = listtransaction.Address
			requestData["confirmationtimestamp"] = listtransaction.Timereceived
			requestData["value"] = listtransaction.Amount
			requestData["transactionid"] = listtransaction.Txid
			requestData["confirmations"] = listtransaction.Confirmations
			requestData["walletaddress"] = true

			outputsArr = append(outputsArr, requestData)
		}
	}

	var jbuf []byte
	resptmp := make(map[string]interface{})
	resptmp["errcode"] = 0
	resptmp["cointype"] = "LTC"
	resptmp["outputs"] = outputsArr

	if jbuf, err = json.Marshal(resptmp); err != nil {
		LOG("ERROR", "lite inner json error:%v", err.Error())
		return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, INNER_ERR, getErrMsg(INNER_ERR, nil))
	}
	return jbuf, nil

}

//Sent an amount from an account to a lite address.
func (m Lite) TransferAccounts(parm interface{}) ([]byte, error) {

	pjs := parm.(*simplejson.Json)
	amountTmp := pjs.Get("amount").MustString()
	destination := pjs.Get("address").MustString()

	if len(amountTmp) == 0 {
		LOG("ERROR", "lite amount illeagal")
		return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, AMOUNT_ERR, getErrMsg(AMOUNT_ERR, nil))
	}
	f, err := strconv.ParseFloat(amountTmp, 64)
	if err != nil {
		LOG("ERROR", "lite inner json error:%v", err.Error())
		return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, INNER_ERR, getErrMsg(INNER_ERR, nil))
	}

	amount := float64(f)

	transactionid, err := liteOp.WalleSendFrom("", destination, amount, 1, "", "")
	if err != nil {
		LOG("ERROR", "lite Send coin error:%v", err.Error())
		return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, SENDCOINS_ERR, getErrMsg(SENDCOINS_ERR, nil))
	}

	if len(transactionid) != 0 {
		var jbuf []byte
		resp := make(map[string]interface{})
		resp["transactionid"] = []string{transactionid}
		resp["errcode"] = 0
		resp["msg"] = "ok"

		if jbuf, err = json.Marshal(resp); err != nil {
			LOG("ERROR", "lite inner json error:%v", err.Error())
			return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, INNER_ERR, getErrMsg(INNER_ERR, nil))
		}
		return jbuf, nil
	}
	LOG("ERROR", "lite transactionid is null error:%v", err.Error())
	return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, INNER_ERR, getErrMsg(INNER_ERR, nil))
}

//create a new lite address&key for receiving payments
func (m Lite) CreateAddress(parm interface{}) ([]byte, error) {

	addr, err := liteOp.WalletNewAddr()
	if err != nil {
		LOG("ERROR", "lite  get new address error:%v", err.Error())
		return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, GETNEWADDRESS_ERR, getErrMsg(GETNEWADDRESS_ERR, nil))
	}
	if len(addr) != 0 {
		//get key by the address
		key, err := liteOp.WalletDumpPrivKey(addr)

		if err != nil {
			LOG("ERROR", "lite dump private key error:%v", err.Error())
			return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, DUMPPRIVKEY_ERR, getErrMsg(DUMPPRIVKEY_ERR, nil))
		}
		if len(key) != 0 {

			var jbuf []byte
			resp := make(map[string]interface{})
			resp["address"] = addr
			resp["privkey"] = key
			resp["errcode"] = 0

			if jbuf, err = json.Marshal(resp); err != nil {
				LOG("ERROR", "lite inner json error:%v", err.Error())
				return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, INNER_ERR, getErrMsg(INNER_ERR, nil))
			}
			return jbuf, nil
		}
		LOG("ERROR", "lite privkey is null error:%v")
		return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, INNER_ERR, getErrMsg(INNER_ERR, nil))
	}
	LOG("ERROR", "lite address is null error:%v")
	return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, INNER_ERR, getErrMsg(INNER_ERR, nil))
}

func (m Lite) getwalletinfo() bool {
	fmt.Println("WalletInfo:")
	m.walletstatus = false
	for {
		if m.walletstatus == false {
			resp, err := liteOp.WalletInfo()
			if err != nil {
				LOG("ERROR", "WalletInfo error:%v", err.Error())
				time.Sleep(time.Second * 1)
				continue
			}

			LOG("DEBUG", "WalletInfo resp:%v", string(resp))
			m.walletstatus = true
		}

		time.Sleep(time.Second)
	}
}

//config rpc from lite.toml
func (m Lite) initconfig() bool {
	if _, err := tml.DecodeFile("./src/lite/lite.toml", &LiteConfig); err != nil {
		fmt.Println(err)
		return false
	}

	fmt.Printf("%+v\n", LiteConfig)

	User := LiteConfig.LiteRpc.Literpcuser
	Password := LiteConfig.LiteRpc.Literpcpassword
	Host := LiteConfig.LiteRpc.Litehost
	Port := LiteConfig.LiteRpc.Liteport

	liteOp.InitRpcConf(User, Password, Host, Port)

	initLiteErr()

	f, err := strconv.ParseFloat(LiteConfig.Limit.Fee, 64)
	if err != nil {
		LOG("ERROR", "lite inner json error:%v", err.Error())
		return false
	}

	fee := float64(f)
	liteOp.WalletSetTxFee(fee)
	return true
}

func (m Lite) ChainHeight(parm interface{}) ([]byte, error) {

	blockcount, err := liteOp.WalleGetBlockCount()
	if err != nil {
		LOG("ERROR", "lite  get current block count error:%v", err.Error())
		return nil, fmt.Errorf(`{"errcode":%v,"mesage":"%s"}`, GETCLOCKCOUNT_ERR, getErrMsg(GETCLOCKCOUNT_ERR, nil))
	}

	sendstr := fmt.Sprintf(`{"errcode":%v,"height":%v}`, OK, blockcount)
	return []byte(sendstr), nil
}
//add by hyb 
func (m Lite) ListUnspent(parm interface{}) ([]byte, error) {
	return nil, nil
}

func (m Lite) SendRawTransaction(parm interface{}) ([]byte, error) {

	pjs := parm.(*simplejson.Json)
	transdata := pjs.Get("data").MustString()

	if len(transdata) == 0 {
		return nil, fmt.Errorf(`{"cointype":"LTC","errcode":%v,"msg":"%v"}`, PARM_ERR, getErrMsg(PARM_ERR, []byte("hexstring is null")))
	}
	transactionhex, err := liteOp.SendRawTransaction(transdata)
	if err != nil {
		LOG("ERROR", " SendRawTransaction err:%s", err.Error())
		return nil, fmt.Errorf(`{"cointype":"LTC","errcode":%v,"msg":"%s"}`, SEND_RAW_TRANSACTION_ERR, getErrMsg(SEND_RAW_TRANSACTION_ERR, nil))
	}

	var jbuf []byte
	resp := make(map[string]interface{})
	resp["cointype"] = "LTC"
	resp["transactionids"] = transactionhex
	resp["errcode"] = 0
	resp["msg"] = "OK"

	if jbuf, err = json.Marshal(resp); err != nil {
		LOG("ERROR", "inner json error:%v", err.Error())
		return nil, fmt.Errorf(`{"cointype":"LTC","errcode":%v,"msg":"%s"}`, INNER_ERR, getErrMsg(INNER_ERR, nil))
	}
	return jbuf, nil
}

func init() {
	Register(NameLite, &Lite{})
}
